@@ -12,11 +12,17 @@
 #
 ##############################################################################
 """Various test-support utility functions
-
-$Id$
 """
 
-import BaseHTTPServer
+try:
+    # Python 3
+    from http.server    import HTTPServer, BaseHTTPRequestHandler
+    from urllib.request import urlopen
+except ImportError:
+    # Python 2
+    from BaseHTTPServer import HTTPServer, BaseHTTPRequestHandler
+    from urllib2        import urlopen
+
 import errno
 import logging
 import os
@@ -28,30 +34,18 @@ import socket
 import subprocess
 import sys
 import tempfile
-import textwrap
 import threading
 import time
-import urllib2
 
 import zc.buildout.buildout
 import zc.buildout.easy_install
 from zc.buildout.rmtree import rmtree
 
+print_ = zc.buildout.buildout.print_
+
 fsync = getattr(os, 'fsync', lambda fileno: None)
 is_win32 = sys.platform == 'win32'
 
-# Only some unixes allow scripts in shebang lines:
-script_in_shebang = is_win32
-if sys.platform == 'linux2':
-    f = subprocess.Popen('uname -r', shell=True, stdout=subprocess.PIPE).stdout
-    r = f.read().strip()
-    f.close()
-    r = tuple(map(int, re.match(r'\d+(\.\d+)*', r).group(0).split('.')))
-    if r >= (2, 6, 27, 9):
-        # http://www.in-ulm.de/~mascheck/various/shebang/
-        script_in_shebang = True
-
-
 setuptools_location = pkg_resources.working_set.find(
     pkg_resources.Requirement.parse('setuptools')).location
 
@@ -62,21 +56,20 @@ def cat(dir, *names):
         and os.path.exists(path+'-script.py')
         ):
         path = path+'-script.py'
-    print open(path).read(),
+    print_(open(path).read(), end='')
 
 def ls(dir, *subs):
     if subs:
         dir = os.path.join(dir, *subs)
-    names = os.listdir(dir)
-    names.sort()
+    names = sorted(os.listdir(dir))
     for name in names:
         if os.path.isdir(os.path.join(dir, name)):
-            print 'd ',
+            print_('d ', end=' ')
         elif os.path.islink(os.path.join(dir, name)):
-            print 'l ',
+            print_('l ', end=' ')
         else:
-            print '- ',
-        print name
+            print_('- ', end=' ')
+        print_(name)
 
 def mkdir(*path):
     os.mkdir(os.path.join(*path))
@@ -105,143 +98,66 @@ def clean_up_pyc(*path):
         filename += 'c' # .py -> .pyc
     for path in (
         os.path.join(base, filename),
-        os.path.join(base, '__pycache__', filename),
+        os.path.join(base, '__pycache__'),
         ):
-        if os.path.exists(path):
+        if os.path.isdir(path):
+            rmdir(path)
+        elif os.path.exists(path):
             remove(path)
 
 ## FIXME - check for other platforms
 MUST_CLOSE_FDS = not sys.platform.startswith('win')
 
-def system(command, input=''):
-    env = dict(os.environ)
-    env['COLUMNS'] = '80'
+def system(command, input='', with_exit_code=False):
     p = subprocess.Popen(command,
                          shell=True,
                          stdin=subprocess.PIPE,
                          stdout=subprocess.PIPE,
                          stderr=subprocess.PIPE,
-                         close_fds=MUST_CLOSE_FDS,
-                         env=env,
-                         )
+                         close_fds=MUST_CLOSE_FDS)
     i, o, e = (p.stdin, p.stdout, p.stderr)
     if input:
-        i.write(input)
+        i.write(input.encode())
     i.close()
     result = o.read() + e.read()
     o.close()
     e.close()
-    return result
-
-def call_py(interpreter, cmd, flags=None):
-    if sys.platform == 'win32':
-        args = ['"%s"' % arg for arg in (interpreter, flags, cmd) if arg]
-        args.insert(-1, '"-c"')
-        return system('"%s"' % ' '.join(args))
-    else:
-        cmd = repr(cmd)
-        return system(
-            ' '.join(arg for arg in (interpreter, flags, '-c', cmd) if arg))
+    output = result.decode()
+    if with_exit_code:
+        # Use the with_exit_code=True parameter when you want to test the exit
+        # code of the command you're running.
+        output += 'EXIT CODE: %s' % p.wait()
+    return output
 
 def get(url):
-    return urllib2.urlopen(url).read()
+    return str(urlopen(url).read().decode())
 
-def _runsetup(setup, executable, *args):
+def _runsetup(setup, *args):
     if os.path.isdir(setup):
         setup = os.path.join(setup, 'setup.py')
-    d = os.path.dirname(setup)
-
     args = list(args)
     args.insert(0, '-q')
-    env = dict(os.environ)
-    if executable == sys.executable:
-        env['PYTHONPATH'] = setuptools_location
-    # else pass an executable that has setuptools! See testselectingpython.py.
-
     here = os.getcwd()
     try:
-        os.chdir(d)
-        p = subprocess.Popen(
-            [executable, setup] + args,
-            stdout=subprocess.PIPE, stderr=subprocess.STDOUT,
-            close_fds=MUST_CLOSE_FDS, env=env)
-        out = p.stdout.read()
-        if p.wait():
-            print out
-
+        os.chdir(os.path.dirname(setup))
+        zc.buildout.easy_install.call_subprocess(
+            [sys.executable, setup] + args,
+            env=dict(os.environ, PYTHONPATH=setuptools_location))
         if os.path.exists('build'):
             rmtree('build')
     finally:
         os.chdir(here)
 
 def sdist(setup, dest):
-    _runsetup(setup, sys.executable, 'sdist', '-d', dest, '--formats=zip')
+    _runsetup(setup, 'sdist', '-d', dest, '--formats=zip')
 
-def bdist_egg(setup, executable, dest):
-    _runsetup(setup, executable, 'bdist_egg', '-d', dest)
-
-def sys_install(setup, dest):
-    _runsetup(setup, sys.executable, 'install', '--install-purelib', dest,
-              '--install-scripts', dest,
-              '--record', os.path.join(dest, '__added_files__'),
-              '--single-version-externally-managed')
-
-def find_python(version):
-    env_friendly_version = ''.join(version.split('.'))
-
-    e = os.environ.get('PYTHON%s' % env_friendly_version)
-    if e is not None:
-        return e
-    if is_win32:
-        e = '\Python%s\python.exe' % env_friendly_version
-        if os.path.exists(e):
-            return e
+def bdist_egg(setup, executable, dest=None):
+    # Backward compat:
+    if dest is None:
+        dest = executable
     else:
-        cmd = 'python%s -c "import sys; print sys.executable"' % version
-        p = subprocess.Popen(cmd,
-                             shell=True,
-                             stdin=subprocess.PIPE,
-                             stdout=subprocess.PIPE,
-                             stderr=subprocess.STDOUT,
-                             close_fds=MUST_CLOSE_FDS)
-        i, o = (p.stdin, p.stdout)
-        i.close()
-        e = o.read().strip()
-        o.close()
-        if os.path.exists(e):
-            return e
-        cmd = 'python -c "import sys; print \'%s.%s\' % sys.version_info[:2]"'
-        p = subprocess.Popen(cmd,
-                             shell=True,
-                             stdin=subprocess.PIPE,
-                             stdout=subprocess.PIPE,
-                             stderr=subprocess.STDOUT,
-                             close_fds=MUST_CLOSE_FDS)
-        i, o = (p.stdin, p.stdout)
-        i.close()
-        e = o.read().strip()
-        o.close()
-        if e == version:
-            cmd = 'python -c "import sys; print sys.executable"'
-            p = subprocess.Popen(cmd,
-                                shell=True,
-                                stdin=subprocess.PIPE,
-                                stdout=subprocess.PIPE,
-                                stderr=subprocess.STDOUT,
-                                close_fds=MUST_CLOSE_FDS)
-            i, o = (p.stdin, p.stdout)
-            i.close()
-            e = o.read().strip()
-            o.close()
-            if os.path.exists(e):
-                return e
-
-    raise ValueError(
-        "Couldn't figure out the executable for Python %(version)s.\n"
-        "Set the environment variable PYTHON%(envversion)s to the location\n"
-        "of the Python %(version)s executable before running the tests."
-        % {'version': version,
-           'envversion': env_friendly_version})
+        assert executable == sys.executable, (executable, sys.executable)
+    _runsetup(setup, 'bdist_egg', '-d', dest)
 
 def wait_until(label, func, *args, **kw):
     if 'timeout' in kw:
@@ -256,64 +172,27 @@ def wait_until(label, func, *args, **kw):
         time.sleep(0.01)
     raise ValueError('Timed out waiting for: '+label)
 
-def get_installer_values():
-    """Get the current values for the easy_install module.
+class TestOptions(zc.buildout.buildout.Options):
 
-    This is necessary because instantiating a Buildout will force the
-    Buildout's values on the installer.
+    def initialize(self):
+        pass
 
-    Returns a dict of names-values suitable for set_installer_values."""
-    names = ('default_versions', 'download_cache', 'install_from_cache',
-             'prefer_final', 'include_site_packages',
-             'allowed_eggs_from_site_packages', 'use_dependency_links',
-             'allow_picked_versions', 'always_unzip'
-            )
-    values = {}
-    for name in names:
-        values[name] = getattr(zc.buildout.easy_install, name)()
-    return values
+class Buildout(zc.buildout.buildout.Buildout):
 
-def set_installer_values(values):
-    """Set the given values on the installer."""
-    for name, value in values.items():
-        getattr(zc.buildout.easy_install, name)(value)
+    def __init__(self):
+        zc.buildout.buildout.Buildout.__init__(
+            self, '', [('buildout', 'directory', os.getcwd())])
 
-def make_buildout(executable=None):
-    """Make a buildout that uses this version of zc.buildout."""
-    # Create a basic buildout.cfg to avoid a warning from buildout.
-    open('buildout.cfg', 'w').write(
-        "[buildout]\nparts =\n"
-        )
-    # Get state of installer defaults so we can reinstate them (instantiating
-    # a Buildout will force the Buildout's defaults on the installer).
-    installer_values = get_installer_values()
-    # Use the buildout bootstrap command to create a buildout
-    config = [
-        ('buildout', 'log-level', 'WARNING'),
-        # trick bootstrap into putting the buildout develop egg
-        # in the eggs dir.
-        ('buildout', 'develop-eggs-directory', 'eggs'),
-        ]
-    if executable is not None:
-        config.append(('buildout', 'executable', executable))
-    zc.buildout.buildout.Buildout(
-        'buildout.cfg', config,
-        user_defaults=False,
-        ).bootstrap([])
-    # Create the develop-eggs dir, which didn't get created the usual
-    # way due to the trick above:
-    os.mkdir('develop-eggs')
-    # Reinstate the default values of the installer.
-    set_installer_values(installer_values)
+    Options = TestOptions
 
 def buildoutSetUp(test):
 
     test.globs['__tear_downs'] = __tear_downs = []
     test.globs['register_teardown'] = register_teardown = __tear_downs.append
 
-    installer_values = get_installer_values()
+    prefer_final = zc.buildout.easy_install.prefer_final()
     register_teardown(
-        lambda: set_installer_values(installer_values)
+        lambda: zc.buildout.easy_install.prefer_final(prefer_final)
         )
 
     here = os.getcwd()
@@ -350,7 +229,6 @@ def buildoutSetUp(test):
     zc.buildout.easy_install.default_index_url = 'file://'+tmp
     os.environ['buildout-testing-index-url'] = (
         zc.buildout.easy_install.default_index_url)
-    os.environ.pop('PYTHONPATH', None)
 
     def tmpdir(name):
         path = os.path.join(base, name)
@@ -360,7 +238,27 @@ def buildoutSetUp(test):
     sample = tmpdir('sample-buildout')
 
     os.chdir(sample)
-    make_buildout()
+
+    # Create a basic buildout.cfg to avoid a warning from buildout:
+    open('buildout.cfg', 'w').write(
+        "[buildout]\nparts =\n"
+        )
+
+    # Use the buildout bootstrap command to create a buildout
+    zc.buildout.buildout.Buildout(
+        'buildout.cfg',
+        [('buildout', 'log-level', 'WARNING'),
+         # trick bootstrap into putting the buildout develop egg
+         # in the eggs dir.
+         ('buildout', 'develop-eggs-directory', 'eggs'),
+         ]
+        ).bootstrap([])
+
+
+
+    # Create the develop-eggs dir, which didn't get created the usual
+    # way due to the trick above:
+    os.mkdir('develop-eggs')
 
     def start_server(path):
         port, thread = _start_server(path, name=path)
@@ -368,52 +266,6 @@ def buildoutSetUp(test):
         register_teardown(lambda: stop_server(url, thread))
         return url
 
-    def make_py(initialization=''):
-        """Returns paths to new executable and to its site-packages.
-        """
-        buildout = tmpdir('executable_buildout')
-        site_packages_dir = os.path.join(buildout, 'site-packages')
-        mkdir(site_packages_dir)
-        old_wd = os.getcwd()
-        os.chdir(buildout)
-        make_buildout()
-        # Normally we don't process .pth files in extra-paths.  We want to
-        # in this case so that we can test with setuptools system installs
-        # (--single-version-externally-managed), which use .pth files.
-        initialization = (
-            ('import sys\n'
-             'import site\n'
-             'known_paths = set(sys.path)\n'
-             'site_packages_dir = %r\n'
-             'site.addsitedir(site_packages_dir, known_paths)\n'
-            ) % (site_packages_dir,)) + initialization
-        initialization = '\n'.join(
-            '  ' + line for line in initialization.split('\n'))
-        install_develop(
-            'zc.recipe.egg', os.path.join(buildout, 'develop-eggs'))
-        install_develop(
-            'z3c.recipe.scripts', os.path.join(buildout, 'develop-eggs'))
-        write('buildout.cfg', textwrap.dedent('''\
-            [buildout]
-            parts = py
-            include-site-packages = false
-            exec-sitecustomize = false
-
-            [py]
-            recipe = z3c.recipe.scripts
-            interpreter = py
-            initialization =
-            %(initialization)s
-            extra-paths = %(site-packages)s
-            eggs = setuptools
-            ''') % {
-                'initialization': initialization,
-                'site-packages': site_packages_dir})
-        system(os.path.join(buildout, 'bin', 'buildout'))
-        os.chdir(old_wd)
-        return (
-            os.path.join(buildout, 'bin', 'py'), site_packages_dir)
-
     cdpaths = []
     def cd(*path):
         path = os.path.join(*path)
@@ -433,7 +285,6 @@ def buildoutSetUp(test):
         tmpdir = tmpdir,
         write = write,
         system = system,
-        call_py = call_py,
         get = get,
         cd = cd, uncd = uncd,
         join = os.path.join,
@@ -442,18 +293,20 @@ def buildoutSetUp(test):
         start_server = start_server,
         buildout = os.path.join(sample, 'bin', 'buildout'),
         wait_until = wait_until,
-        make_py = make_py,
+        print_ = print_,
         clean_up_pyc = clean_up_pyc,
         ))
 
+    zc.buildout.easy_install.prefer_final(prefer_final)
+
 def buildoutTearDown(test):
     for f in test.globs['__tear_downs']:
         f()
 
-class Server(BaseHTTPServer.HTTPServer):
+class Server(HTTPServer):
 
     def __init__(self, tree, *args):
-        BaseHTTPServer.HTTPServer.__init__(self, *args)
+        HTTPServer.__init__(self, *args)
         self.tree = os.path.abspath(tree)
 
     __run = True
@@ -464,29 +317,34 @@ class Server(BaseHTTPServer.HTTPServer):
     def handle_error(self, *_):
         self.__run = False
 
-class Handler(BaseHTTPServer.BaseHTTPRequestHandler):
+class Handler(BaseHTTPRequestHandler):
 
     Server.__log = False
 
     def __init__(self, request, address, server):
         self.__server = server
         self.tree = server.tree
-        BaseHTTPServer.BaseHTTPRequestHandler.__init__(
-            self, request, address, server)
+        BaseHTTPRequestHandler.__init__(self, request, address, server)
 
     def do_GET(self):
         if '__stop__' in self.path:
             raise SystemExit
 
+        def k():
+            self.send_response(200)
+            out = '<html><body>k</body></html>\n'.encode()
+            self.send_header('Content-Length', str(len(out)))
+            self.send_header('Content-Type', 'text/html')
+            self.end_headers()
+            self.wfile.write(out)
+
         if self.path == '/enable_server_logging':
             self.__server.__log = True
-            self.send_response(200)
-            return
+            return k()
 
         if self.path == '/disable_server_logging':
             self.__server.__log = False
-            self.send_response(200)
-            return
+            return k()
 
         path = os.path.abspath(os.path.join(self.tree, *self.path.split('/')))
         if not (
@@ -496,7 +354,7 @@ class Handler(BaseHTTPServer.BaseHTTPRequestHandler):
             ):
             self.send_response(404, 'Not Found')
             #self.send_response(200)
-            out = '<html><body>Not Found</body></html>'
+            out = '<html><body>Not Found</body></html>'.encode()
             #out = '\n'.join(self.tree, self.path, path)
             self.send_header('Content-Length', str(len(out)))
             self.send_header('Content-Type', 'text/html')
@@ -507,14 +365,13 @@ class Handler(BaseHTTPServer.BaseHTTPRequestHandler):
         self.send_response(200)
         if os.path.isdir(path):
             out = ['<html><body>\n']
-            names = os.listdir(path)
-            names.sort()
+            names = sorted(os.listdir(path))
             for name in names:
                 if os.path.isdir(os.path.join(path, name)):
                     name += '/'
                 out.append('<a href="%s">%s</a><br>\n' % (name, name))
             out.append('</body></html>\n')
-            out = ''.join(out)
+            out = ''.join(out).encode()
             self.send_header('Content-Length', str(len(out)))
             self.send_header('Content-Type', 'text/html')
         else:
@@ -528,13 +385,14 @@ class Handler(BaseHTTPServer.BaseHTTPRequestHandler):
                 self.send_header('Content-Type', 'application/x-gzip')
             else:
                 self.send_header('Content-Type', 'text/html')
+
         self.end_headers()
 
         self.wfile.write(out)
 
     def log_request(self, code):
         if self.__server.__log:
-            print '%s %s %s' % (self.command, code, self.path)
+            print_('%s %s %s' % (self.command, code, self.path))
 
 def _run(tree, port):
     server_address = ('localhost', port)
@@ -552,7 +410,7 @@ def get_port():
                 return port
         finally:
             s.close()
-    raise RuntimeError, "Can't find port"
+    raise RuntimeError("Can't find port")
 
 def _start_server(tree, name=''):
     port = get_port()
@@ -567,7 +425,7 @@ def start_server(tree):
 
 def stop_server(url, thread=None):
     try:
-        urllib2.urlopen(url+'__stop__')
+        urlopen(url+'__stop__')
     except Exception:
         pass
     if thread is not None:
@@ -583,7 +441,8 @@ def wait(port, up):
             s.close()
             if up:
                 break
-        except socket.error, e:
+        except socket.error:
+            e = sys.exc_info()[1]
             if e[0] not in (errno.ECONNREFUSED, errno.ECONNRESET):
                 raise
             s.close()
@@ -596,7 +455,7 @@ def wait(port, up):
             raise SystemError("Couln't stop server")
 
 def install(project, destination):
-    if not isinstance(destination, basestring):
+    if not isinstance(destination, str):
         destination = os.path.join(destination.globs['sample_buildout'],
                                    'eggs')
 
@@ -616,7 +475,7 @@ def install(project, destination):
              ).write(dist.location)
 
 def install_develop(project, destination):
-    if not isinstance(destination, basestring):
+    if not isinstance(destination, str):
         destination = os.path.join(destination.globs['sample_buildout'],
                                    'develop-eggs')
 
@@ -633,14 +492,10 @@ def _normalize_path(match):
             path = path[1:]
     return '/' + path.replace(os.path.sep, '/')
 
-if sys.platform == 'win32':
-    sep = r'[\\/]' # Windows uses both sometimes.
-else:
-    sep = re.escape(os.path.sep)
 normalize_path = (
     re.compile(
-        r'''[^'" \t\n\r!]+%(sep)s_[Tt][Ee][Ss][Tt]_%(sep)s([^"' \t\n\r]+)'''
-        % dict(sep=sep)),
+        r'''[^'" \t\n\r]+\%(sep)s_[Tt][Ee][Ss][Tt]_\%(sep)s([^"' \t\n\r]+)'''
+        % dict(sep=os.path.sep)),
     _normalize_path,
     )
 
@@ -650,7 +505,29 @@ normalize_script = (
     re.compile('(\n?)-  ([a-zA-Z_.-]+)-script.py\n-  \\2.exe\n'),
     '\\1-  \\2\n')
 
+if sys.version_info > (2, ):
+    normalize___pycache__ = (
+        re.compile('(\n?)d  __pycache__\n'), '\\1')
+else:
+    normalize___pycache__ = (
+        re.compile('(\n?)-  \S+\.pyc\n'), '\\1')
+
 normalize_egg_py = (
     re.compile('-py\d[.]\d(-\S+)?.egg'),
     '-pyN.N.egg',
     )
+
+normalize_exception_type_for_python_2_and_3 = (
+    re.compile(r'^(\w+\.)*([A-Z][A-Za-z0-9]+Error: )'),
+    '\2')
+
+not_found = (re.compile(r'Not found: [^\n]+/(\w|\.)+/\r?\n'), '')
+
+# Setuptools now pulls in dependencies when installed.
+adding_find_link = (re.compile(r"Adding find link '[^']+'"
+                               r" from setuptools .*\r?\n"), '')
+
+ignore_not_upgrading = (
+    re.compile(
+    'Not upgrading because not running a local buildout command.\n'
+    ), '')
